/******************************************************************/
/*                                                                */
/* Copyright (C) 1994 - 2005, Micromuse, Ltd.                     */
/* All Rights Reserved                                            */
/*                                                                */
/*----------------------------------------------------------------*/
/*                                                                */
/* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE                    */
/* The copyright notice above does not evidence any               */
/* actual or intended publication of such source code.            */
/*                                                                */
/* RESTRICTED RIGHTS:                                             */
/*                                                                */
/* This file may have been supplied under a license.              */
/* It may be used, disclosed, and/or copied only as permitted     */
/* under such license agreement. Any copy must contain the        */
/* above copyright notice and this restricted rights notice.      */
/* Use, copying, and/or disclosure of the file is strictly        */
/* prohibited unless otherwise provided in the license agreement. */
/*                                                                */
/******************************************************************/

/**
********************************************************************************
*
* PropertyValue()
*
* Return the value of a property as a string.
*
* @param propName Name of the property to return the value of.
*
* @return The property value, or an empty string if the property was not found.
*
********************************************************************************
*/
function PropertyValue(propName)
{
    var answer = "";

    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    var prop = propMgr.GetProp(propName);
    if (prop)
    {
        answer = prop.value;
    }

    return answer;
}

/**
********************************************************************************
*
* Log()
*
* Log a message to the installer log file.
*
* @param level The lowest level at which the message should be logged.
*              0 - Errors
*              1 - Warnings
*              2 - Info
*              3 - Debug
*
* @param msg The message to be logged.
*
********************************************************************************
*/
function Log(level, msg)
{
    try
    {
        var logger = Components.classes["@micromuse.com/NCILog;1"].
            getService(Components.interfaces.nsINCILog);

        switch (level)
        {
            case 0:
                logger.error(msg);
                break;
            case 1:
                logger.warning(msg);
                break;
            case 2:
                logger.info(msg);
                break;
            case 3:
                logger.debug(msg);
                break;
            default:
                break;
        }
    }
    catch(e)
    {
        // Logging failure must not affect functionality, so fail silently.
    }
}

/**
********************************************************************************
*
* GetNcHome()
*
* Get the value of the NCHOME environment variable.
*
* @return The value of the NCHOME environment variable.
*
********************************************************************************
*/
function GetNcHome()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    return propMgr.GetProp("$NCHOME").value;
}

/**
********************************************************************************
*
* ReadFile
*
* Read an existing file.  The caller should ensure the file exists before
* calling this function.
*
* @param file The nsIFile object to be read from.
*
* @return The entire file contents as a string.
*
********************************************************************************
*/
function ReadFile(file)
{
    var result = "";

    var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].
        createInstance(Components.interfaces.nsIFileInputStream);
    var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].
        createInstance(Components.interfaces.nsIScriptableInputStream);

    try
    {
        fstream.init(file, 1, 0, 0);
        sstream.init(fstream);

        var bytes = fstream.available();
        while (bytes > 0)
        {
            result += sstream.read(bytes);

            bytes = fstream.available();
        }
        sstream.close();
        fstream.close();
    }
    catch(e)
    {
        Log(0, e);
    }

    return result;
}

/**
********************************************************************************
*
* GetFullPlatformPath()
*
* Convert the stub file paths found in our filelist files to a full path that is
* correct for the current platform.  On Windows, this will also involve
* replacing forward slashes with backslashes.
*
* For example, if $NCHOME is set to /opt/netcool then for an input of
* precision/aoc/default/Device.aoc, this function will return
* /opt/netcool/precision/aoc/default/Device.aoc.
*
* @param relocName Stub path name to expand and make platform specific.
*
* @return The expanded, platform specific path name.
*
********************************************************************************
*/
function GetFullPlatformPath(relocName)
{
    var result = GetNcHome() + "/" + relocName;

    if (ncipackage.shortArch == "win32")
    {
        result = result.replace(/\//g, "\\");
    }

    return result;
}

/**
********************************************************************************
*
* GetPlatformTempDirPath()
*
* Given the name of a temporary file, return the full path to it in a suitable
* temporary directory.
*
* If possible, this temporary directory will be $NCHOME/precision/install_temp.
* If this directory doesn't exist and cannot be created, the temporary directory
* will be $NCHOME.
*
* @param tempFileName Stub path name to expand and make platform specific.
*
* @return The expanded, platform specific path name.
*
********************************************************************************
*/
function GetPlatformTempDirPath(relocName)
{
    var result = "";

    var dirName = GetFullPlatformPath("precision/install_temp");

    var ok = true;

    try
    {
        var dir = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        dir.initWithPath(dirName);

        if (dir.exists())
        {
            if (!dir.isDirectory() || !dir.isWritable())
            {
                //
                // The path we want to use for the temporary directory exists,
                // but it's not a directory that we can write to.
                //
                ok = false;
            }
        }
        else
        {
            //
            // Directory does not exist, so try to make it.
            //
            var parentDirName = GetFullPlatformPath("precision");

            //
            // Try to create the parent directory too, but don't worry about errors
            //
            try
            {
                var parentDir = Components.classes["@mozilla.org/file/local;1"].
                    createInstance(Components.interfaces.nsILocalFile);
                parentDir.initWithPath(parentDirName);

                parentDir.create(1, 0777);
            }
            catch(e)
            {
            }

            dir.create(1, 0777);

            if (!dir.exists())
            {
                ok = false;
            }
            else
            {
                Log(2, "Successfully created directory " + dirName);
            }
        }
    }
    catch(e)
    {
        Log(0, e);
        ok = false;
    }

    if (ok)
    {
        result = GetFullPlatformPath("precision/install_temp/" + relocName);
    }
    else
    {
        Log(1, dirName + " was not an existing directory and could not be created");
        result = GetFullPlatformPath(relocName);
    }

    return result;
}

/**
********************************************************************************
*
* GetFullInstalledPath()
*
* Given a stub file path (like the ones found in our filelist files) for a
* default config file, return the full path where the used version of the config
* file should go.  This will be correct for the current platform, i.e. on
* Windows, forward slashes will be replaced with backslashes.
*
* For example, if $NCHOME is set to /opt/netcool then for an input of
* precision/aoc/default/Device.aoc, this function will return
* /opt/netcool/precision/aoc/Device.aoc.
*
* @param relocName Stub path name to expand and make platform specific.
*
* @return The expanded, platform specific path name.
*
********************************************************************************
*/
function GetFullInstalledPath(relocName)
{
    return GetFullPlatformPath(relocName.replace(/default\//, ""));
}

/**
********************************************************************************
*
* GetInstalledDirectoryName()
*
* Given a stub file path (like the ones found in our filelist files) for a
* default config file, return the full path to the directory where the used
* version of the config file should go.  This will be correct for the current
* platform, i.e. on Windows, forward slashes will be replaced with backslashes.
*
* For example, if $NCHOME is set to /opt/netcool then for an input of
* precision/aoc/default/Device.aoc, this function will return
* /opt/netcool/precision/aoc.
*
* @param relocName Stub path name to expand and make platform specific.
*
* @return The expanded, platform specific path name.
*
********************************************************************************
*/
function GetInstalledDirectoryName(relocName)
{
    var tmp = relocName.replace(/default\//, "");

    return GetFullPlatformPath(tmp.substring(0, tmp.lastIndexOf("/")));
}

/**
********************************************************************************
*
* GetSignature()
*
* Get a signature for a file.  This can be used to tell if a file has changed.
* The signature consists of the length of the file, followed by a hash code
* computed from the contents of the file.
*
* @param fileName Absolute path to the file to get the signature for.
*
* @return The signature for the given file.
*
********************************************************************************
*/
function GetSignature(fileName)
{
    var answer = "";

    var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(fileName);

    if (file.exists())
    {
        answer += file.fileSize;

        var contents = ReadFile(file);

        var highBits = 15 << 28;
        var tmp = 0;
        var hash = 0;
        for (var count = 0; count < contents.length; ++count)
        {
            hash <<= 4;
            hash += contents.charCodeAt(count);
            tmp = hash & highBits;
            if (tmp != 0)
            {
                hash = (hash ^ (tmp >>> 24)) & ~highBits;
            }
        }

        answer += "#" + hash;

        Log(3, "Signature for file " + fileName + " is " + answer);
    }
    else
    {
        answer = "0#0"
    }

    return answer;
}

/**
********************************************************************************
*
* DoFilesDiffer()
*
* Check whether two files differ.  Any differences, including whitespace, will
* be treated as a difference.
*
* If the first file doesn't exist, then this is NOT counted as a difference.
*
* If the second file doesn't exist, but the first one does, then this IS counted
* as a difference.
*
* @param fileNameA Absolute path to the first file to compare.
*
* @param fileNameB Absolute path to the second file to compare.
*
* @return true if both files exist and are different; otherwise false.
*
********************************************************************************
*/
function DoFilesDiffer(fileNameA, fileNameB)
{
    var answer = false;

    var fileA = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    fileA.initWithPath(fileNameA);

    if (fileA.exists())
    {
        var fileB = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
        fileB.initWithPath(fileNameB);

        if (fileB.exists())
        {
            var contentsA = ReadFile(fileA);
            var contentsB = ReadFile(fileB);

            if (contentsA != contentsB)
            {
                Log(3, fileNameA + " and " + fileNameB + " are different");

                answer = true;
            }
            else
            {
                Log(3, fileNameA + " and " + fileNameB + " are the same");
            }
        }
        else
        {
            Log(3, fileNameB + " does not exist");

            answer = true;
        }
    }
    else
    {
        Log(3, fileNameA + " does not exist");
    }

    return answer;
}

/**
********************************************************************************
*
* ConfigFilePreCheck()
*
* Work through the list of config files, checking whether any of them are
* different from their default versions.  If so, create a file called
* $NCHOME/package_name.user.tmp containing the array indexes of the changed
* files.
*
* Additionally, create a file containing signatures of the current default
* config files.  This is used to ensure the user is only warned when this
* installation actually changes a file they have changed themselves.
*
* @param configFiles An array (which may be empty) containing a list of the
*                    configuration files associated with the current package.
*
********************************************************************************
*/
function ConfigFilePreCheck(configFiles)
{
    var userFilename = GetPlatformTempDirPath(ncipackage.name + ".user.tmp");
    var signatureFilename = GetPlatformTempDirPath(ncipackage.name + ".signature.tmp");

    var userContents = "";
    var signatureContents = "";

    try
    {
        var userFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        userFile.initWithPath(userFilename);

        var signatureFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        signatureFile.initWithPath(signatureFilename);

        var userStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        userStream.init(userFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

        var signatureStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        signatureStream.init(signatureFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

        var installedFileName = "";
        var defaultFileName = "";

        for (var count = 0; count < configFiles.length; ++count)
        {
            installedFileName = GetFullInstalledPath(configFiles[count]);
            defaultFileName = GetFullPlatformPath(configFiles[count]);

            //
            // Get the length and hash code for the current default file.
            //
            if (signatureContents.length > 0)
            {
                signatureContents += ",";
            }
            signatureContents += GetSignature(defaultFileName);

            //
            // Has the user modified the file?
            //
            if (DoFilesDiffer(installedFileName, defaultFileName))
            {
                if (userContents.length > 0)
                {
                    userContents += ",";
                }
                userContents += count;
            }
        }

        userStream.write(userContents, userContents.length);
        userStream.close();

        signatureStream.write(signatureContents, signatureContents.length);
        signatureStream.close();
    }
    catch(e)
    {
        Log(0, e);
        Log(0, "Unable to write " + userFilename + " and " + signatureFilename);
    }
}

/**
********************************************************************************
*
* PopulateSignatureArray()
*
* Populate an array with the signatures of the previous config files.  The
* returned array is indexed using the index of the config file array.
*
* @param file The file to read that contains the signatures of the previous
*             config files.
*
* @return The populated array of previous config signatures.
*
********************************************************************************
*/
function PopulateSignatureArray(file)
{
    var result = [];

    var contents = ReadFile(file);
    if (contents.length > 0)
    {
        result = contents.split(",");
    }

    return result;
}

/**
********************************************************************************
*
* PopulateChangedArray()
*
* Populate an array with the indexes of those config files that have changed.
* The returned array is indexed using the index of the changed config files that
* are read from the given file.
*
* @param file The file to read that contains the indexes of the changed config
*             files.
*
* @return The populated array of changed indexes.
*
********************************************************************************
*/
function PopulateChangedArray(file)
{
    var result = new Array;

    var contents = ReadFile(file);
    if (contents.length > 0)
    {
        var contentsArray = contents.split(",");

        for (var count = 0; count < contentsArray.length; ++count)
        {
            result[Number(contentsArray[count])] = true;
        }
    }

    return result;
}

/**
********************************************************************************
*
* OverwriteFile()
*
* Overwrite one file with the contents of another.
*
* @param targetFileName Name of file to overwrite.
*
* @param sourceFileName Name of source file.
*
********************************************************************************
*/
function OverwriteFile(targetFileName, sourceFileName)
{
    var sourceFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    sourceFile.initWithPath(sourceFileName);

    try
    {
        if (sourceFile.exists())
        {
            var contents = ReadFile(sourceFile);

            var targetFile = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            targetFile.initWithPath(targetFileName);

            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(targetFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
            foStream.write(contents, contents.length);
            foStream.close();
        }
        else
        {
            Log(0, "Cannot update file " + targetFileName +
                   " because " + sourceFileName + " does not exist");
        }
    }
    catch(e)
    {
        Log(0, "Unable to write to file " + targetFileName);
    }
}

/**
********************************************************************************
*
* ProcessDefaultConfigFiles()
*
* Work through the list of config files, checking whether the default was
* different to the installed file before installation.  If so, inform the user.
* If not, copy the default file to the install directory.
*
* @param configFiles An array (which may be empty) containing a list of the
*                    configuration files associated with the current package.
*
********************************************************************************
*/
function ProcessDefaultConfigFiles(configFiles)
{
    var userFilename = GetPlatformTempDirPath(ncipackage.name + ".user.tmp");
    var signatureFilename = GetPlatformTempDirPath(ncipackage.name + ".signature.tmp");

    try
    {
        var userFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        userFile.initWithPath(userFilename);

        var changedArray = [];

        if (userFile.exists())
        {
            changedArray = PopulateChangedArray(userFile);

            userFile.remove(false);
        }

        var signatureFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        signatureFile.initWithPath(signatureFilename);

        var signatures = [];

        if (signatureFile.exists())
        {
            signatures = PopulateSignatureArray(signatureFile);

            signatureFile.remove(false);
        }

        var copyIt = true;
        var installedFile = "";
        var defaultFile = "";

        for (var count = 0; count < configFiles.length; ++count)
        {
            installedFile = GetFullInstalledPath(configFiles[count]);
            defaultFile = GetFullPlatformPath(configFiles[count]);

            copyIt = true;

            if (changedArray[count])
            {
                //
                // File may have been changed by the user.
                // If so, tell the user to merge it themselves.
                // But first double check that the newly
                // installed default is different to the
                // existing version.
                //
                if (DoFilesDiffer(installedFile, defaultFile))
                {
                    copyIt = false;

                    //
                    // The next possibility is that although
                    // the user has changed the file, this install
                    // isn't changing the default version.  If this
                    // is the case then we won't copy the default,
                    // but the user doesn't need to be warned.
                    //
                    if (signatures[count] != GetSignature(defaultFile))
                    {
                        Log(1, defaultFile + " has changed, and " + installedFile
                             + " contains customisations not present in it"
                             + " - please merge any required changes by hand");
                    }
                    else
                    {
                        Log(2, installedFile + " contains customisations not present in "
                             + defaultFile + ", but no changes were made to " + defaultFile
                             + " on this installation");
                    }
                }
            }

            if (copyIt)
            {
                //
                // File has not been changed by the user.
                // Copy the default to the install directory.
                //
                OverwriteFile(installedFile, defaultFile);
            }
        }
    }
    catch(e)
    {
        Log(0, e);
    }
}

/**
********************************************************************************
*
* ProcessAgents()
*
* Work through the list of config files, checking whether each one is an agent
* definition file.  If it is, run the ncp_agent_registrar process to register
* it.
*
* @param configFiles An array (which may be empty) containing a list of the
*                    configuration files associated with the current package.
*
* @param action The action to apply -- this should match one of the possible
*               parameters of ncp_agent_registrar: either register or remove.
*
********************************************************************************
*/
function ProcessAgents(configFiles, action)
{
    try
    {
        var strippedFileName = "";
        var agentsToRegister = "";

        for (var count = 0; count < configFiles.length; ++count)
        {
            if (configFiles[count].match(/disco.*\.agnt$/))
            {
                strippedFileName = configFiles[count].substr(configFiles[count].lastIndexOf("/") + 1);

                if (agentsToRegister.length > 0)
                {
                    agentsToRegister += ",";
                }

                agentsToRegister += strippedFileName;
            }
        }

        if (agentsToRegister.length > 0)
        {
            Log(2, "About to " + action + " the following discovery agents: " + agentsToRegister);

            //
            // Set up platform specific variables
            //
            var separator = "/";
            var topLine = "#!/bin/sh";
            var precisionHomeEnv = "PRECISION_HOME=" + GetNcHome() + "/precision\n"
                             + "export PRECISION_HOME";
            var registrarPath = GetNcHome() + "/precision/bin/ncp_agent_registrar";

            if (ncipackage.shortArch == "win32")
            {
                separator = "\\";
                topLine = "@echo off";
                precisionHomeEnv = "set PRECISION_HOME=" + GetNcHome() + "\\precision\n"
                             + "set NCHOME=" + GetNcHome() + "\n"
                             + "path " + GetNcHome() + "\\platform\\win32\\bin;%Path%"
                registrarPath = GetNcHome()
                            + "\\precision\\platform\\win32\\bin\\ncp_agent_registrar.exe";
            }

            try
            {
                //
                // Now create a script to run "ncp_agent_registrar -<ACTION> <AGENTS>"
                //
                var scriptFileName = GetNcHome() + separator + "batch_config.bat";

                var scriptFile = Components.classes["@mozilla.org/file/local;1"].
                    createInstance(Components.interfaces.nsILocalFile);
                scriptFile.initWithPath(scriptFileName);

                var scriptContents = topLine + "\n"
                                   + precisionHomeEnv + "\n"
                                   + "\"" + registrarPath + "\" -" + action + " \""
                                   + agentsToRegister + "\" > \""
                                   + GetNcHome() + separator + "log" + separator
                                   + "precision" + separator
                                + "ncp_agent_registrar.log\" 2>&1\n";

                var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                    createInstance(Components.interfaces.nsIFileOutputStream);
                foStream.init(scriptFile, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
                foStream.write(scriptContents, scriptContents.length);
                foStream.close();

                //
                // Now run the script.
                //
                var process = Components.classes["@mozilla.org/process/util;1"].
                    createInstance(Components.interfaces.nsIProcess);
                process.init(scriptFile);

                var args = [];
                process.run(true, args, args.length);

                //
                // Finally, clean up the temporary file.
                //
                scriptFile.remove(false);
            }
            catch(e)
            {
                Log(0, "Unable to " + action + " the following discovery agents: " + agentsToRegister);
            }
        }
    }
    catch(e)
    {
        Log(0, e);
    }
}

/**
********************************************************************************
*
* RenameObsoleteFile()
*
* Rename an obsolete config file.
*
* @param fileName The name of the file that is now obsolete.
*
********************************************************************************
*/
function RenameObsoleteFile(fileName)
{
    try
    {
        var obsoleteFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        obsoleteFile.initWithPath(fileName);

        if (obsoleteFile.exists())
        {
            var newFileName = fileName + ".obsolete";

            OverwriteFile(newFileName, fileName);

            obsoleteFile.remove(false);

            Log(1, "Renamed obsolete config file " + fileName + " to " + newFileName);
        }
    }
    catch(e)
    {
        Log(0, e);
    }
}

/**
********************************************************************************
*
* RemoveObsoleteConfigFiles()
*
* Work through the list of obsolete config files, renaming them, even if there
* are domain specific versions.
*
* If this was not done, the obsolete config files could interfere with the
* operation of the system.
*
* @param obsoleteConfigFiles An array (which may be empty) containing a list of
*                            obsolete configuration files that used to be part
*                            of the current package.
*
********************************************************************************
*/
function RemoveObsoleteConfigFiles(obsoleteConfigFiles)
{
    //
    // First unregister any obsolete agents.
    //
    ProcessAgents(obsoleteConfigFiles, "remove");

    try
    {
        var copyIt = true;
        var fileName = "";
        var filenameRegex = "";
        var directoryName = "";
        var directory = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        var entries = null;
        var file = null;
        var pattern = null;

        var options = "";
        var separator = "/";
        if (ncipackage.shortArch == "win32")
        {
            options = "i";
            separator = "\\";
        }

        for (var count = 0; count < obsoleteConfigFiles.length; ++count)
        {
            //
            // This is the config file name without any directory.
            //
            fileName = obsoleteConfigFiles[count].replace(/.*\//, "");
            filenameRegex = "^" + fileName.replace(/\./, "\\..*") + "$";
            pattern = new RegExp(filenameRegex, options);

            //
            // There could be domain specific versions of the file.  Search for
            // them using a regular expression.
            //
            directoryName = GetInstalledDirectoryName(obsoleteConfigFiles[count]);

            directory.initWithPath(directoryName);

            if (directory.exists() && directory.isDirectory())
            {
                try
                {
                    entries = directory.directoryEntries;

                    while (entries.hasMoreElements())
                    {
                        file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
                        if (file)
                        {
                            if (pattern.test(file.leafName))
                            {
                                RenameObsoleteFile(directoryName + separator + file.leafName);
                            }
                        }
                    }
                }
                catch(e)
                {
                    Log(0, e);
                }
            }
        }
    }
    catch(e)
    {
        Log(0, e);
    }
}

/**
********************************************************************************
*
* ConfigFilePostInstall()
*
* Perform any processing associated with config files installed by this package.
* At present, this conists of:
* 1) Checking whether a config file has been changed both by us and the user,
*    and thus requires a merge
* 2) Running the agent registrar to register each discovery agent as it is
*    installed
*
* @param configFiles An array (which may be empty) containing a list of the
*                    configuration files associated with the current package.
*
********************************************************************************
*/
function ConfigFilePostInstall(configFiles)
{
    ProcessDefaultConfigFiles(configFiles);

    ProcessAgents(configFiles, "register");
}

